﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Data.Common;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Threading;
using System.Linq;
using System.Windows.Controls.Primitives;
using System.Windows.Media;
using System.Globalization;
using Infragistics.Controls.Editors;
using Infragistics.Controls.Maps;
using Infragistics.Controls.Charts;
using System.Configuration;
using System.Reflection;
using System.Text;
using System.Windows.Markup;
using System.Text.RegularExpressions;

namespace WeatherMapPerfExample
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private List<LatLonForcast> listResults;

        public MainWindow()
        {
            InitializeComponent();
            this.DataContext = this;
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            CustomPaletteColorScale colorScale = generateColorPalette();
            listResults = GenData();

            TriangulationSource triangluation = TriangulationSource.Create(listResults.Count, getXY, getValue);
            GeographicScatterAreaSeries geoSeries = new GeographicScatterAreaSeries();
            geoSeries.ColorScale = colorScale;
            geoSeries.LongitudeMemberPath = "Point.X";
            geoSeries.LatitudeMemberPath = "Point.Y";
            geoSeries.TriangleVertexMemberPath1 = "V1";
            geoSeries.TriangleVertexMemberPath2 = "V2";
            geoSeries.TriangleVertexMemberPath3 = "V3";
            geoSeries.ItemsSource = triangluation.Points;
            geoSeries.TrianglesSource = triangluation.Triangles;

            GeoMap.Series.Add(geoSeries);
        }

        private List<LatLonForcast> GenData()
        {
            Random rnd = new Random();
            List<LatLonForcast> gridPoints = new List<LatLonForcast>();
            // Rough US Grid
            for(double lat = 23.00; lat <= 49.00; lat += 0.1)
            {
                for (double lon = -66.00; lon >= -124.00; lon -= 0.1)
                {
                    LatLonForcast loc = new LatLonForcast();
                    loc.Latitude = lat;
                    loc.Longitude = lon;
                    loc.Value = rnd.Next(-20, 106);
                    gridPoints.Add(loc);
                }
            }

            return gridPoints;
        }

        public Point getXY(int index)
        {
            return new Point(listResults[index].Longitude, listResults[index].Latitude);
        }

        public float getValue(int index)
        {
            return Convert.ToSingle(listResults[index].Value.Value);
        }

        // http://stackoverflow.com/questions/2011832/generate-color-gradient-in-c-sharp
        private CustomPaletteColorScale generateColorPalette()
        {
            CustomPaletteColorScale colorScale = new CustomPaletteColorScale();
            //colorScale.MinimumValue = 0.05;
            //colorScale.MinimumValue = 1.75;
            colorScale.Palette = new ObservableCollection<Color>();

            byte rMax = Colors.Pink.R;
            byte rMin = Colors.Cyan.R;
            byte gMax = Colors.Pink.G;
            byte gMin = Colors.Cyan.G;
            byte bMax = Colors.Pink.B;
            byte bMin = Colors.Cyan.B;
            int size = 50;
            for (int i = 0; i < size; i++)
            {
                byte rAverage = (byte)(rMin + (int)((rMax - rMin) * i / size));
                byte gAverage = (byte)(gMin + (int)((gMax - gMin) * i / size));
                byte bAverage = (byte)(bMin + (int)((bMax - bMin) * i / size));
                colorScale.Palette.Add(Color.FromArgb(200, rAverage, gAverage, bAverage));
            }

            size = 25;
            for (int i = 0; i < size; i++)
            {
                colorScale.Palette.Add(Colors.Transparent);
            }

            rMax = Colors.Yellow.R;
            rMin = Colors.Red.R;
            gMax = Colors.Yellow.G;
            gMin = Colors.Red.G;
            bMax = Colors.Yellow.B;
            bMin = Colors.Red.B;
            size = 30;
            for (int i = 0; i < size; i++)
            {
                byte rAverage = (byte)(rMin + (int)((rMax - rMin) * i / size));
                byte gAverage = (byte)(gMin + (int)((gMax - gMin) * i / size));
                byte bAverage = (byte)(bMin + (int)((bMax - bMin) * i / size));
                colorScale.Palette.Add(Color.FromArgb(150, rAverage, gAverage, bAverage));
            }

            return colorScale;
        }
    }

    public class LatLonForcast
    {
        public double Latitude { get; set; }

        public double Longitude { get; set; }

        public double? Value { get; set; }
    }
}
